@@ -3,7 +3,7 @@
  * Authors:
  *   Chris Toshok <toshok@ximian.com>
  *
- * Copyright (C) 2003 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #ifndef __E_CONTACT_H__
@@ -170,7 +170,7 @@ typedef enum {
 	E_CONTACT_IM_YAHOO,   	 /* Multi-valued */
 	E_CONTACT_IM_MSN,     	 /* Multi-valued */
 	E_CONTACT_IM_ICQ,     	 /* Multi-valued */
-       
+
 	E_CONTACT_WANTS_HTML,    /* boolean field */
 
 	/* fields used for describing contact lists.  a contact list
@@ -186,8 +186,6 @@ typedef enum {
 	/* Security Fields */
 	E_CONTACT_X509_CERT,     /* structured field (EContactCert) */
 
-	/*New Fields Added to the Enum and Padding for future*/
-	
 	E_CONTACT_IM_GADUGADU_HOME_1,  /* Synthetic string field */
 	E_CONTACT_IM_GADUGADU_HOME_2,  /* Synthetic string field */
 	E_CONTACT_IM_GADUGADU_HOME_3,  /* Synthetic string field */
@@ -196,7 +194,10 @@ typedef enum {
 	E_CONTACT_IM_GADUGADU_WORK_3,  /* Synthetic string field */
 
 	E_CONTACT_IM_GADUGADU,   /* Multi-valued */
-	
+
+	E_CONTACT_GEO,	/* structured field (EContactGeo) */
+
+	E_CONTACT_TEL, /* list of strings */
 
 	E_CONTACT_FIELD_LAST,
 	E_CONTACT_FIELD_FIRST        = E_CONTACT_UID,
@@ -222,6 +223,11 @@ typedef struct {
 	char *suffixes;
 } EContactName;
 
+typedef struct {
+	double latitude;
+	double longitude;
+} EContactGeo;
+
 typedef enum {
 	E_CONTACT_PHOTO_TYPE_INLINED,
 	E_CONTACT_PHOTO_TYPE_URI
@@ -316,6 +322,9 @@ void                    e_contact_name_free        (EContactName *name);
 GType                   e_contact_photo_get_type   (void);
 void                    e_contact_photo_free       (EContactPhoto *photo);
 
+GType			e_contact_geo_get_type	   (void);
+void			e_contact_geo_free	   (EContactGeo *geo);
+
 GType                   e_contact_cert_get_type    (void);
 void                    e_contact_cert_free        (EContactCert *cert);
 
