@@ -3,7 +3,7 @@
   FILE: 
   CREATOR: eric 25 June 2000
   
-  $Id: testmime.c 620 2007-08-28 15:09:06Z rburton $
+  $Id: testmime.c 633 2008-02-18 07:26:45Z pchen $
   $Locker$
     
  The contents of this file are subject to the Mozilla Public License
@@ -207,10 +207,13 @@ int main(int argc, char* argv[]) {
 
     if(opt.normal == 1){
 	icalcomponent *c;
+	char *obj;
 
 	c = icalmime_parse(read_stream,f);
 	
-	printf("%s\n",icalcomponent_as_ical_string(c));
+	obj = icalcomponent_as_ical_string (c);
+	printf("%s\n",obj);
+	free (obj);
 
 	icalcomponent_free(c);
 
@@ -245,6 +248,7 @@ int main(int argc, char* argv[]) {
 	
 
 	for(j=0; j<opt.count; j++){
+	    char *obj;
 
 	    srand(j);
 	    memset(buf,0,size*2);
@@ -270,7 +274,9 @@ int main(int argc, char* argv[]) {
 
 	    c = icalmime_parse(icalparser_string_line_generator,&d);
 
-	    printf("%s\n",icalcomponent_as_ical_string(c));
+	    obj = icalcomponent_as_ical_string (c);
+	    printf("%s\n", obj);
+	    free (obj);
 
 	    icalcomponent_free(c);
 	    
