@@ -5,7 +5,7 @@
  * Authors: Bertrand Guiheneuf <bertrand@helixcode.com>
  *	    Michael Zucchi <notzed@ximian.com>
  *
- * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU Lesser General Public 
@@ -109,6 +109,9 @@ struct _CamelFolder {
 
 	guint32 folder_flags;
 	guint32 permanent_flags;
+
+	/* Future ABI expansion */
+	gpointer later[4];
 };
 
 #define CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY (1<<0)
@@ -174,7 +177,9 @@ typedef struct {
 	GPtrArray * (*get_uids)       (CamelFolder *folder);
 	void (*free_uids)             (CamelFolder *folder,
 				       GPtrArray *array);
-
+	
+	void (* sort_uids) (CamelFolder *folder, GPtrArray *uids);
+	
 	GPtrArray * (*get_summary)    (CamelFolder *folder);
 	void (*free_summary)          (CamelFolder *folder,
 				       GPtrArray *summary);
@@ -203,8 +208,13 @@ typedef struct {
 	void     (*freeze)    (CamelFolder *folder);
 	void     (*thaw)      (CamelFolder *folder);
 	gboolean (*is_frozen) (CamelFolder *folder);
-
+	
 	CamelFolderQuotaInfo * (*get_quota_info) (CamelFolder *folder);
+	guint32	(*count_by_expression) (CamelFolder *, const char *, CamelException *);
+	void (*sync_message)  (CamelFolder *folder,
+                               const char *uid, 
+                               CamelException *ex);
+        GPtrArray * (*get_uncached_uids)(CamelFolder *, GPtrArray * uids, CamelException *);
 } CamelFolderClass;
 
 /* Standard Camel function */
@@ -265,7 +275,7 @@ void		   camel_folder_set_message_user_tag  (CamelFolder *folder,
 						       const char *uid,
 						       const char *name,
 						       const char *value);
-#endif
+#endif /* CAMEL_DISABLE_DEPRECATED */
 
 
 
@@ -297,18 +307,27 @@ void               camel_folder_free_summary          (CamelFolder *folder,
 CamelMimeMessage * camel_folder_get_message           (CamelFolder *folder, 
 						       const char *uid, 
 						       CamelException *ex);
+void               camel_folder_sync_message          (CamelFolder *folder, 
+						       const char *uid, 
+						       CamelException *ex);
 #define camel_folder_delete_message(folder, uid) \
 	camel_folder_set_message_flags (folder, uid, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN)
 
 GPtrArray *        camel_folder_get_uids              (CamelFolder *folder);
 void               camel_folder_free_uids             (CamelFolder *folder,
 						       GPtrArray *array);
+GPtrArray *        camel_folder_get_uncached_uids     (CamelFolder *,
+                                                       GPtrArray * uids,
+                                                       CamelException *);
+void               camel_folder_sort_uids             (CamelFolder *folder,
+						       GPtrArray *uids);
 
 /* search api */
 gboolean           camel_folder_has_search_capability (CamelFolder *folder);
 GPtrArray *	   camel_folder_search_by_expression  (CamelFolder *folder, const char *expr, CamelException *ex);
 GPtrArray *	   camel_folder_search_by_uids	      (CamelFolder *folder, const char *expr, GPtrArray *uids, CamelException *ex);
 void		   camel_folder_search_free	      (CamelFolder *folder, GPtrArray *result);
+guint32		   camel_folder_count_by_expression   (CamelFolder *folder, const char *expression, CamelException *ex);
 
 /* summary info */
 CamelMessageInfo *camel_folder_get_message_info		(CamelFolder *folder, const char *uid);
