@@ -229,8 +229,11 @@ groupwise_populate_details_from_item (CamelMimeMessage *msg, EGwItem *item)
 		time_t actual_time;
 		int offset = 0;
 		dtstring = e_gw_item_get_creation_date (item);
-		time = e_gw_connection_get_date_from_string (dtstring);
-		actual_time = camel_header_decode_date (ctime(&time), NULL);
+		if (dtstring) {
+				time = e_gw_connection_get_date_from_string (dtstring);
+				actual_time = camel_header_decode_date (ctime(&time), NULL);
+		} else
+				actual_time = (time_t) 0;
 		camel_mime_message_set_date (msg, actual_time, offset);
 	}
 }
@@ -922,7 +925,13 @@ update_update (CamelSession *session, CamelSessionThreadMsg *msg)
 	done = FALSE;
 	m->slist = NULL;
 
-	while (!done && !camel_application_is_exiting) {
+	while (!done) {
+
+		if (camel_application_is_exiting) {
+				CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
+				return ;
+		}
+
 		item_list = NULL;
 		status = e_gw_connection_get_all_mail_uids (m->cnc, m->container_id, cursor, FALSE, READ_CURSOR_MAX_IDS, position, &item_list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -1191,10 +1200,6 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		if (gw_store->current_folder != folder) {
 			gw_store->current_folder = folder;
 		}
-
-		if (list) {
-			gw_update_cache (folder, list, ex, FALSE);
-		}
 		
 		if (check_all && !is_proxy) {
 				EGwContainer *container;
@@ -1205,8 +1210,15 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 
 				if (camel_folder_summary_count (folder->summary) == e_gw_container_get_total_count (container))
 						check_all = FALSE;
+
+				folder->summary->unread_count = e_gw_container_get_unread_count (container);
+				folder->summary->visible_count = e_gw_container_get_total_count (container);
 				g_object_unref (container);
 		}
+
+		if (list) {
+			gw_update_cache (folder, list, ex, FALSE);
+		}
 	}
 
 
@@ -1270,7 +1282,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 	int folder_needs_caching;
 
-	camel_object_get (folder, NULL, CAMEL_OFFLINE_FOLDER_ARG_SYNC_OFFLINE, &folder_needs_caching, NULL);
+	camel_object_get (folder, NULL, CAMEL_OFFLINE_FOLDER_SYNC_OFFLINE, &folder_needs_caching, NULL);
 
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
@@ -1342,6 +1354,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
 				mi->info.content->type = camel_content_type_new ("multipart", "mixed");	
 			}
+			mi->info.flags = 0;
 		}
 		
 		rk = e_gw_item_get_recurrence_key (item);
@@ -1356,16 +1369,12 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 		item_status = e_gw_item_get_item_status (item);
 		if (item_status & E_GW_ITEM_STAT_READ)
-			status_flags |= CAMEL_MESSAGE_SEEN;
+			mi->info.flags |= CAMEL_MESSAGE_SEEN;
 		else 
 			mi->info.flags &= ~CAMEL_MESSAGE_SEEN;
 
 		if (item_status & E_GW_ITEM_STAT_REPLIED)
-			status_flags |= CAMEL_MESSAGE_ANSWERED;
-		if (exists) 
-			mi->info.flags |= status_flags;
-		else 
-			mi->info.flags = status_flags;
+			mi->info.flags |= CAMEL_MESSAGE_ANSWERED;
 
 		priority = e_gw_item_get_priority (item);
 		if (priority && !(g_ascii_strcasecmp (priority,"High"))) {
@@ -1446,12 +1455,17 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 				time_t time;
 				time_t actual_time;
 				temp_date = e_gw_item_get_creation_date (item);
-				time = e_gw_connection_get_date_from_string (temp_date);
-				actual_time = camel_header_decode_date (ctime(&time), NULL);
+				if (temp_date) {
+						/* Creation date can be returned as null for auto-generated meetings */
+						time = e_gw_connection_get_date_from_string (temp_date);
+						actual_time = camel_header_decode_date (ctime(&time), NULL);
+				} else
+					actual_time = (time_t) 0;
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		}
 
+		mi->info.dirty = TRUE;
 		if (exists) {
 			camel_folder_change_info_change_uid (changes, mi->info.uid);
 			camel_message_info_free (pmi);
@@ -1459,12 +1473,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 			mi->info.uid = camel_pstring_strdup (e_gw_item_get_id(item));
 			mi->info.size = e_gw_item_get_mail_size (item);	
 			mi->info.subject = camel_pstring_strdup(e_gw_item_get_subject(item));
-			mi->info.dirty = TRUE;
 			
-			folder->summary->visible_count ++;
-			if (!(mi->info.flags & CAMEL_MESSAGE_SEEN))
-					folder->summary->unread_count ++;
-
 			camel_folder_summary_add (folder->summary,(CamelMessageInfo *)mi);
 			camel_folder_change_info_add_uid (changes, mi->info.uid);
 			camel_folder_change_info_recent_uid (changes, mi->info.uid);
@@ -1664,8 +1673,11 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 				time_t time;
 				time_t actual_time;
 				temp_date = e_gw_item_get_creation_date (item);
-				time = e_gw_connection_get_date_from_string (temp_date);
-				actual_time = camel_header_decode_date (ctime(&time), NULL);
+				if (temp_date) {
+						time = e_gw_connection_get_date_from_string (temp_date);
+						actual_time = camel_header_decode_date (ctime(&time), NULL);
+				} else
+						actual_time = (time_t) 0;
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		}
@@ -1774,7 +1786,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 									gsize len_iter = 0;
 									char *temp = NULL;
 	
-									temp = g_base64_decode(t_attach, &len_iter);
+									temp = (char *) g_base64_decode(t_attach, &len_iter);
 									gstr = g_string_append_len (gstr, temp, len_iter);
 									g_free (temp);
 									len += len_iter;
@@ -1918,7 +1930,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 								gsize len_iter = 0;
 								char *temp = NULL;
 
-								temp = g_base64_decode(t_attach, &len_iter);
+								temp = (char *) g_base64_decode(t_attach, &len_iter);
 								gstr = g_string_append_len (gstr, temp, len_iter);
 								g_free (temp);
 								len += len_iter;
@@ -2680,7 +2692,21 @@ convert_to_task (EGwItem *item, char **str, int *len)
 	g_string_append_printf (gstr, "UID:%s\n",e_gw_item_get_icalid (item));
 	g_string_append_printf (gstr, "DTSTART:%s\n",e_gw_item_get_start_date (item));
 	g_string_append_printf (gstr, "SUMMARY:%s\n", e_gw_item_get_subject (item));
-	g_string_append_printf (gstr, "DESCRIPTION:%s\n", e_gw_item_get_message (item));
+
+	temp = e_gw_item_get_message (item);
+	if (temp) {
+		g_string_append(gstr, "DESCRIPTION:");
+		while (*temp) {
+			if (*temp == '\n')
+				g_string_append(gstr, "\\n");
+			else
+				g_string_append_c(gstr, *temp);
+			temp++;
+		}
+		g_string_append(gstr, "\n");	
+	}
+	temp = NULL;
+
 	g_string_append_printf (gstr, "DTSTAMP:%s\n", e_gw_item_get_creation_date (item));
 	g_string_append_printf (gstr, "X-GWMESSAGEID:%s\n", e_gw_item_get_id (item));
 	g_string_append_printf (gstr, "X-GWSHOW-AS:BUSY\n");
@@ -2735,6 +2761,7 @@ convert_to_note (EGwItem *item, char **str, int *len)
 	EGwItemOrganizer *org = NULL;
 	GString *gstr = g_string_new (NULL);
 	char **tmp = NULL;
+	const char *temp = NULL;
 
 	tmp = g_strsplit (e_gw_item_get_id (item), "@", -1);
 
@@ -2744,7 +2771,21 @@ convert_to_note (EGwItem *item, char **str, int *len)
 	g_string_append_printf (gstr, "UID:%s\n",e_gw_item_get_icalid (item));
 	g_string_append_printf (gstr, "DTSTART:%s\n",e_gw_item_get_start_date (item));
 	g_string_append_printf (gstr, "SUMMARY:%s\n", e_gw_item_get_subject (item));
-	g_string_append_printf (gstr, "DESCRIPTION:%s\n", e_gw_item_get_message (item));
+
+	temp = e_gw_item_get_message (item);
+	if (temp) {
+		g_string_append(gstr, "DESCRIPTION:");
+		while (*temp) {
+			if (*temp == '\n')
+				g_string_append(gstr, "\\n");
+			else
+				g_string_append_c(gstr, *temp);
+			temp++;
+		}
+		g_string_append(gstr, "\n");	
+	}
+	temp = NULL;
+
 	g_string_append_printf (gstr, "DTSTAMP:%s\n", e_gw_item_get_creation_date (item));
 	g_string_append_printf (gstr, "X-GWMESSAGEID:%s\n", e_gw_item_get_id (item));
 	g_string_append_printf (gstr, "X-GWRECORDID:%s\n", tmp[0]);
