@@ -1,10 +1,10 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-storage.c
  *
- * Copyright (C) 2000, 2001 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,10 +12,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Ettore Perazzoli
  */
@@ -27,10 +27,9 @@
 #include "e-storage.h"
 
 #include "e-folder-tree.h"
-#include "e-shell-marshal.h"
 
 #include <glib/gi18n-lib.h>
-#include <libedataserver/e-util.h>
+#include <libedataserver/e-data-server-util.h>
 
 #include <string.h>
 
@@ -101,7 +100,7 @@ folder_changed_cb (EFolder *folder,
 		p = strrchr (path, '/');
 		if (p && p != path) {
 			char *name;
-			
+
 			name = g_strndup (path, p - path);
 			folder = e_folder_tree_get_folder (priv->folder_tree, name);
 			g_free (name);
@@ -283,7 +282,7 @@ e_storage_class_init (EStorageClass *class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EStorageClass, new_folder),
 			      NULL, NULL,
-			      e_shell_marshal_NONE__STRING,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
 	signals[UPDATED_FOLDER] =
@@ -292,7 +291,7 @@ e_storage_class_init (EStorageClass *class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EStorageClass, updated_folder),
 			      NULL, NULL,
-			      e_shell_marshal_NONE__STRING,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
 	signals[REMOVED_FOLDER] =
@@ -301,7 +300,7 @@ e_storage_class_init (EStorageClass *class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EStorageClass, removed_folder),
 			      NULL, NULL,
-			      e_shell_marshal_NONE__STRING,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
 }
@@ -624,9 +623,9 @@ get_path_for_physical_uri_foreach (EFolderTree *folder_tree,
  * e_storage_get_path_for_physical_uri:
  * @storage: A storage
  * @physical_uri: A physical URI
- * 
+ *
  * Look for the folder having the specified @physical_uri.
- * 
+ *
  * Return value: The path of the folder having the specified @physical_uri in
  * @storage.  If such a folder does not exist, just return NULL.  The return
  * value must be freed by the caller.
